/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef C_MATRIX_HPP
#define C_MATRIX_HPP
#include <iostream>
#include <fstream>
#include <vector>

namespace p {

#define PXX_ADJ_ELEMENT		unsigned long
#define PXX_ADJ_BITS		(sizeof(PXX_ADJ_ELEMENT) * 8)

// ****************************************************************************
// 
// ****************************************************************************
class adjacency
{

public:
	adjacency();
	adjacency(int n_vertices);
	adjacency(const adjacency &src);
	~adjacency();
	
	void clear();
	void resize(int n_vertices);
	int  vertices() const;
	int  pairs() const;
	int  offset(int i, int j) const;

	adjacency &operator=(const adjacency &src);
	
	void set(bool value);
	void set(int i, int j, bool value);
	void complement(int i, int j);
	void complement();
	bool get(int i, int j) const;
	bool operator() (int i, int j) const;
	
	// Node adjacency lists
	adjacency &pack();
	int  pack_degree(int i) const;
	const int *pack_adj(int i) const;
	
	class it
	{
	public:
		it();
		it(const adjacency &a, int i);
		it(const it &src);
		~it();
		
		int operator*() const;
		
		int operator++();
		int operator++(int);
		
		it &operator=(const it &si);
		
		void of(const adjacency &a, int i);
	private:
		int degree, offset, j;
		const int *adj;
	};
	
	// I/O
	int  read_simple(std::istream &f);
	int  read_simple(const std::string &path);
	int  write_simple(std::ostream &f) const;
	int  write_simple(const std::string &path) const;

private:
	int n, e, ee;
	
	PXX_ADJ_ELEMENT *data;
	int **nlist;
};

std::ostream &operator<<(std::ostream &os, const adjacency &a);

// ****************************************************************************
// 
// ****************************************************************************
class adjacency_pre
{

public:
	adjacency_pre(const adjacency &src);
	~adjacency_pre();

	int  components() const;
	void build_component(int p, adjacency &d, std::vector<int> &orig_idx);

private:
	const adjacency &a;
	int ncomp;
	std::vector<int> comp;
};


// ****************************************************************************
// 
// ****************************************************************************
} // namespace

#endif

