/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PXX_CONSTRAINT_HPP
#define PXX_CONSTRAINT_HPP
#include <string>
#include "dense_vector.hpp"
#include "sparse_vector.hpp"

namespace p {

// **************************************************
class constraint {

public:
	constraint();
	constraint(const svec &row, char sense, double rhs);
	constraint(const constraint &src);
	~constraint();
	
	constraint &operator=(const constraint &src);
	void set_name(const std::string &str);
	void setup(const svec &row, char sense, double rhs);
	
	const char *cstr_name() const;
	
	double lhs(const dvec &x) const;
	double violation(const dvec &x) const;
	double slack(const dvec &x) const;
	double norm() const;
	double dynamism(const tolerance &tol = sparse_tol) const;
	
public:
	std::string name;
	
	svec row;
	char sense;
	double rhs;
};

// **************************************************

} // namespace

#endif

