/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <cassert>
#include "cut_pool.hpp"

namespace p {

// *************************************************************************
// 
// *************************************************************************
cut::cut()
	: age(0)
{
}

cut::cut(const constraint &src)
	: constraint(src), age(0)
{
}

cut::cut(const cut &src)
	: constraint(src), age(src.age)
{
}

cut::cut(const svec &row, char sense, double rhs)
	: constraint(row, sense, rhs), age(0)
{
}

cut::~cut()
{
}

cut &cut::operator=(const cut &src)
{
	assert(tracker.target == NULL);
	
	constraint::operator=(src);
	age = src.age;
	
	return(*this);
}

// *************************************************************************
// 
// *************************************************************************
cut_pool::cut_pool()
{
}

cut_pool::~cut_pool()
{
}

void cut_pool::clear()
{
	pool.clear();
}

cut &cut_pool::add()
{
	pool.push_back(cut());
	
	return(pool.back());
}

cut &cut_pool::add(const constraint &cns)
{
	pool.push_back(cut(cns));
	
	return(pool.back());
}

cut &cut_pool::add(const svec &row, char sense, double rhs)
{
	cut c(row, sense, rhs);
	
	pool.push_back(c);
	
	return(pool.back());
}


// *************************************************************************
// 
// *************************************************************************
} // namespace

