/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PXX_DENSE_VECTOR_HPP
#define PXX_DENSE_VECTOR_HPP
#include <iostream>
#include <vector>

namespace p {

class svec;

class dvec : public std::vector<double> {

public:
	dvec();
	dvec(const dvec &src);
	dvec(int n);
	dvec(int n, const double *val);
	dvec(const svec &src);
	dvec(const std::vector<double> &src);
	
	void copy(int n, const double *val);
	void copy(int n, double val);
	void set(double val);
	void set(int i0, int i1, double val);
	void set(int i0, int i1, const dvec &src, int j0 = 0);
	void set(int i0, int i1, const double *val, int j0 = 0);
	void pad(int n, double val);
	
	double sqnorm() const;
	double norm() const;
	double sum() const;
	
	dvec operator+(const dvec &src) const;
	dvec operator-(const dvec &src) const;
	double operator*(const dvec &src) const;
	double operator*(const svec &src) const;

	dvec operator+() const;
	dvec operator-() const;
	dvec operator+(double scalar) const;
	dvec operator-(double scalar) const;
	dvec operator*(double scalar) const;
	dvec operator/(double scalar) const;

	dvec &operator=(const dvec &src);
	dvec &operator=(const svec &src);
	
	dvec &operator+=(const dvec &src);
	dvec &operator-=(const dvec &src);
	dvec &operator+=(double scalar);
	dvec &operator-=(double scalar);
	dvec &operator*=(double scalar);
	dvec &operator/=(double scalar);
};


std::ostream &operator<<(std::ostream &os, const dvec &v);
std::istream &operator>>(std::istream &is, dvec &v);


} // namespace

#endif
