#include <iostream>
#include <cassert>
#include <string>
#include "pxx/dense_vector.hpp"
#include "pxx/sparse_vector.hpp"
#include "pxx/model.hpp"
#include "pxx/cplex_model.hpp"
#include "pxx/rational.hpp"
#include "pxx/strprintf.hpp"

int main(int argc, char **argv)
{
	// Get input & output path
	if ((argc < 2) || (argc > 3)) {
		std::cout << "Usage: presolve.run {input_file} [output_file]"
			<< std::endl;
		return(0);
	}
	
	std::string fname(argv[1]);
	std::string oname;
	
	// Replace '.mps' with '.pre.mps'
	if (argc >= 3) {
		oname = argv[2];
	} else {
		oname = fname;
		
		size_t dir = oname.rfind("/");
	
		if (dir != std::string::npos)
			oname.erase(0, dir + 1);
	
		size_t mps = oname.rfind(".mps");
	
		if (mps == std::string::npos)
			mps = oname.length();

		oname.insert(mps, ".pre");
	}
	
	// Read problem
	p::cplex_model cpxp;
	
	cpxp.getenv().setintparam(CPX_PARAM_SCRIND, CPX_ON);

	std::cout << fname << ": reading..." << std::endl;

	if (cpxp.readcopyprob(fname.c_str(), NULL)) {
		std::cout << fname << ": failed" << std::endl;
		return(1);
	}
	
	// Presolve problem
	std::cout << fname << ": presolve..." << std::endl;

	cpxp.presolve();
	
	CPXCLPptr prelp = cpxp.getredlp_or_self();
	
	p::cplex_model pre(prelp);
	
	// Add slacks (standard form) and objective offset
	std::cout << fname << ": export..." << std::endl;
	
	p::model std;
	p::elim_ruleset el;
	double offset;
	
	pre.export_model(std);
	offset = pre.getobjoffset();
	
	std::cout << fname << ": standard form (obj offset " << offset
		<< ")..." << std::endl;
	std.standard(el, PXX_STD_INT_SLACKS);
	
	std.obj.push_back(1.0);
	std.lower.push_back(offset);
	std.upper.push_back(offset);
	std.ctype.push_back('C');
	std.cname.push_back("_obj_offs");
	
	std::cout << fname << ": import..." << std::endl;
	pre.import_model(std);
	
	// Write presolved problem
	std::cout << "presolve.run: Writing " << oname << std::endl;
	pre.writeprob(oname.c_str(), NULL);
	
	return(0);
}


