#! /usr/bin/awk -f
#
# Usage: columns.awk {columns}
#
# Print on stdout a subset of the columns of stdin.
#
# The 'columns' field is a comma-separated ordered subset of:
# name,time,two_calls,two_iter,two_iter_max,two_time,two_comp,two_sep,
# gc,gc[1-5]{one|two},u,T1,T2,T3,Q1,Q2,splits
#
# columns:
#  1. problem name
#  2. total time
#  3. two-row separator called
#  4. Q(S) constraints added
#  5. Q(S) constraints added (per sep max)
#  6. two-row separator time
#  7. two-row separator computed
#  8. two-row separator separates
#  9. one-row split called
# 10. one-row split time
# 11. one-row split separates
# 12. final %gc
# 13, 15, 17, 19, 21. %gc after rank 1-5 one-row
# 14, 16, 18, 20, 22. %gc after rank 1-5 one-row + two-row
# 23-29: 23:?, 24:T1, 25:T2, 26:T3, 27:Q1, 28:Q2, 29:Split

BEGIN {
	cn["name"] = 1
	cn["time"] = 2
	cn["two_calls"] = 3
	cn["two_iter"] = 4
	cn["two_iter_max"] = 5
	cn["two_time"] = 6
	cn["two_comp"] = 7
	cn["two_sep"] = 8
	cn["one_calls"] = 9
	cn["one_time"] = 10
	cn["one_sep"] = 11
	cn["gc"] = 12
	cn["gc1one"] = 13
	cn["gc1two"] = 14
	cn["gc2one"] = 15
	cn["gc2two"] = 16
	cn["gc3one"] = 17
	cn["gc3two"] = 18
	cn["gc4one"] = 19
	cn["gc4two"] = 20
	cn["gc5one"] = 21
	cn["gc5two"] = 22
	cn["u"] = 23
	cn["T1"] = 24
	cn["T2"] = 25
	cn["T3"] = 26
	cn["Q1"] = 27
	cn["Q2"] = 28
	cn["splits"] = 29


	#for (i = 1; i < ARGC; i++)
	#	cl[i] = ARGV[i]
	ncl = split(ARGV[1], cl, ",")
	ARGC = 1
}

{
	for (i = 1; i <= ncl; i++) {
		if (cl[i] in cn) {
			printf "%s ", $cn[cl[i]]
		} else {
			printf "? "
		}
	}
	printf "\n"
}

END {
}

