#! /bin/bash
#
# Usage: group.sh {file columns}...
#
# Print on stdout a latex-formatted joined table (with averages)
# of the experiments summarized in 'files'. The join column is the
# first of each file
#
# The 'columns' field is a comma-separated ordered subset of:
#  name,time,two_calls,two_iter,two_iter_max,two_time,two_comp,two_sep,
#  one_calls,one_time,one_sep,gc,gc[1-5]{one|two},u,T1,T2,T3,Q1,Q2,splits
#
#

if (( $# < 2 )); then
	echo "Usage: group.sh {file columns}..."
	echo
	echo "Columns:"
	echo "  name,time,two_calls,two_iter,two_iter_max,two_time,two_comp,two_sep,"
	echo "  one_calls,one_time,one_sep,gc,gc[1-5]{one|two},u,T1,T2,T3,Q1,Q2,splits"
	echo
	exit 0
fi

group=""

while (($#)); do
	group=$(echo "$group"; echo "file: $1"; cat $1 | ./columns.awk $2)
	shift 2
done

echo "$group" | ./join.awk | ./average.awk | ./tabular.awk

