#include <cstdlib>
#include "pxx/strprintf.hpp"
#include "pxx/cplex_model.hpp"
#include "concave.hpp"

// ************************************************************************
// 
// ************************************************************************
double urand(double a, double b)
{
	double r;
	
	r = ((double)rand()) / (double)RAND_MAX;
	
	return( a + r * (b - a) );
}


// ************************************************************************
// 
// ************************************************************************
concave_prob::concave_prob(p::cplex_model &src)
	: prob(src)
{
	prob.fetch_columns();
	
	n = prob.obj.size();
	g.resize(n);
	x.resize(n);
	
	prob.make_LP();
	prob.lpopt();
	prob.getx(&x[0], 0, n - 1);
	prob.getdj(&g[0], 0, n - 1);
}

// ************************************************************************
// 
// ************************************************************************
concave_prob::~concave_prob()
{
}

// ************************************************************************
// 
// ************************************************************************
void concave_prob::opt(objective &obj)
{
	p::dvec x0(x);
	double step;
	int it = 0;
	
	do {
		prob.set_objective(obj.gradient(x0));
		prob.lpopt();
		prob.getx(&x[0], 0, n - 1);
		step = (x - x0).norm();
		std::cout << p::strprintf(
			"concave_prob::opt(): it %d: obj %g; step %g",
			it++, obj.value(x), step) << std::endl;
		x0 = x;
	} while (step > 1e-6);
	
	prob.getdj(&g[0], 0, n - 1);
}

// ************************************************************************
// 
// ************************************************************************

