/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef GRAPH_HPP
#define GRAPH_HPP
#include <iostream>

namespace p {

// **************************************************************************
// 
// **************************************************************************
struct node {
	int value, meta;
	int parent, first_child, prev_sibling, next_sibling;
};


// **************************************************************************
// 
// **************************************************************************
class forest
{
private:
	int n;
	node *f;
	
public:
	forest();
	forest(int nodes);
	forest(const forest &sf);
	~forest();
	
	int  size() const;
	void resize(int nodes);
	
	const node &operator[](int i) const;
	node &operator[](int i);
	
	void singletons();

	bool descending(int i, int j) const;
	bool ascending(int i, int j) const;
	bool linked(int i, int j) const;
	
	int  root(int i) const;
	void reroot(int i);

	void attach(int child, int parent);
	void link(int i, int j);
	void unlink(int i, int j);
	
	void complement(int i);
	void tree_complement(int root);
	void tree_meta(int root, int value);
	
	class it
	{
	public:
		it(const forest &sf, int i0 = 0);
		it(const forest &sf, int root, int i0);
		it(const it &si);
		~it();
		
		it &operator=(const it &si);
		it &operator=(int i0);
		
		int operator*() const;
		int depth() const;

		int operator++();
		int operator++(int);
	
	private:
		const forest &f;
		int first, i, d;
	};
};

std::ostream &operator<<(std::ostream &os, const forest &f);

// **************************************************************************
// 
// **************************************************************************
} // namespace

#endif


