/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <vector>
#include <iostream>
#include "index_tracker.hpp"

namespace p {

// *************************************************************************
// 
// *************************************************************************
index_tracker::index_tracker()
	: target(NULL), index(-1)
{
}

index_tracker::~index_tracker()
{
	if (target) {
		target->clear_tracker(index);
	}
}

// *************************************************************************
// 
// *************************************************************************
index_target::index_target()
{
}

index_target::~index_target()
{
	clear();
}

void index_target::clear()
{
	for (int j = 0; j < (int)tracker.size(); j++)
		clear_tracker(j);
	
	tracker.clear();
}

int index_target::size() const
{
	return(tracker.size());
}

int index_target::add()
{
	int n = tracker.size();
	
	tracker.push_back(NULL);
	
	return(n);
}

void index_target::remove(int i)
{
	clear_tracker(i);
	
	tracker.erase(tracker.begin() + i);
	
	for (int j = i; j < (int)tracker.size(); j++) {
		if (tracker[j]) {
			tracker[j]->index--;
		}
	}
}

void index_target::clear_tracker(int i)
{
	if (tracker[i] == NULL)
		return;
	
	tracker[i]->target = NULL;
	tracker[i]->index = -1;
	tracker[i] = NULL;
}

void index_target::set_tracker(int i, index_tracker *t)
{
	clear_tracker(i);
	
	if (t) {
		t->target = this;
		t->index = i;
	}

	tracker[i] = t;
}

// *************************************************************************
// 
// *************************************************************************

} // namespace
