/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PXX_MODEL_HPP
#define PXX_MODEL_HPP
#include <vector>
#include <list>
#include "dense_vector.hpp"
#include "constraint.hpp"

namespace p {

// ************************************************
#define PXX_INF_BOUND	1e20

#define PXX_STD_INT_SLACKS	1

// ************************************************
class elim_rule {

public:
	void eliminate(constraint &r) const;
public:
	int slack;
	constraint initial;
};


// ************************************************
class elim_ruleset : public std::list<elim_rule> {

public:
	elim_rule &add();
	void eliminate(constraint &r) const;
};

// ************************************************
class model {

public:
	model();
	model(const model &src);
	~model();

	void clear();
	model &operator=(const model &src);

	constraint &add_constraint();
	
	bool int_slack(const constraint &c) const;
	void standard(elim_ruleset &el, int opts);

public:
	// columns
	dvec lower, upper;
	dvec obj;
	std::vector<char> ctype;
	std::vector<std::string> cname;

	// rows
	std::list<constraint> constraints;
	
};

// ************************************************

} // namespace

#endif

