/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <cmath>
#include "onerow.hpp"

namespace p {

// **************************************************************************
// 
// **************************************************************************
onerow::onerow()
{
}

onerow::~onerow()
{
}


// **************************************************************************
// 
// **************************************************************************
int onerow::separate(constraint &c,
	model &q, const dvec &x0, const constraint &c1, int b1,
	const std::string &cname)
{
	c.name = cname;
	c.row.clear();
	c.row.resize(q.obj.size());
	c.sense = 'G';
	c.rhs = 1.0;
	
	double frac_plus = x0[b1] - floor(x0[b1]);
	double frac_minus = 1.0 - frac_plus;
	
	for (int l = 0; l < c1.row.nz(); l++) {
		int j = c1.row.index(l);
		
		if (j == b1)
			continue;
		
		double a = c1.row.value(l);
		double dist_lb = x0[j] - q.lower[j];
		double dist_ub = q.upper[j] - x0[j];
		bool at_upper = (dist_ub < dist_lb);
		
		if (at_upper)
			a = -a;
		
		double alpha = (a >= 0.0) ? (a / frac_plus) : (-a / frac_minus);
		
		if (at_upper) {
			c.row.push_nz(j, -alpha);
			c.rhs -= alpha * q.upper[j];
		} else {
			c.row.push_nz(j, alpha);
			c.rhs += alpha * q.lower[j];
		}
	}
	
	return(0);
}

// **************************************************************************
// 
// **************************************************************************

} // namespace

