/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PXX_TIMER_HPP
#define PXX_TIMER_HPP
#include <sys/time.h>
#include <string>
#include <list>
#include <iostream>

namespace p {

// ************************************************
class timer {

public:
	timer();
	timer(const timer &src);
	~timer();
	
	timer &operator=(const timer &src);
	
	void enable();
	void disable();
	bool enabled() const;
	
	void clear();
	void start();
	void stop();
	
	void shadow_start();
	void shadow_stop();
	
	double elapsed_s() const;
	double total_s() const;
	int calls() const;

private:
	bool on;
	struct timeval total;
	struct timeval begin;
	int nc;
};


std::ostream &operator<<(std::ostream &os, const timer &t);


// ************************************************

} // namespace


#endif


