/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <math.h>
#include "tolerance.hpp"

namespace p {


// *************************************************************************
// 
// *************************************************************************
tolerance::tolerance()
	: tol(1e-8)
{
}

tolerance::tolerance(double tol0)
	: tol(tol0)
{
}

tolerance::tolerance(const tolerance &src)
	: tol(src.tol)
{
}

tolerance::~tolerance()
{
}

tolerance &tolerance::operator=(const tolerance &src)
{
	tol = src.tol;
	
	return(*this);
}

bool tolerance::equal(double a, double b) const
{
	return(fabs(a - b) <= tol);
}

bool tolerance::zero(double a) const
{
	return(fabs(a) <= tol);
}

bool tolerance::nonzero(double a) const
{
	return(fabs(a) > tol);
}

bool tolerance::integral(double a) const
{
	return(fabs(a - floor(a + 0.5)) <= tol);
}

bool tolerance::fractional(double a) const
{
	return(fabs(a - floor(a + 0.5)) > tol);
}

double tolerance::ceil(double a) const
{
	return(::ceil(a - tol));
}

double tolerance::floor(double a) const
{
	return(::floor(a + tol));
}

bool tolerance::inclusive_le(double a, double b) const
{
	return(a - tol <= b);
}

bool tolerance::exclusive_le(double a, double b) const
{
	return(a + tol <= b);
}

bool tolerance::inclusive_ge(double a, double b) const
{
	return(a + tol >= b);
}

bool tolerance::exclusive_ge(double a, double b) const
{
	return(a - tol >= b);
}


// *************************************************************************
// 
// *************************************************************************
tolerance sparse_tol(1e-8);
tolerance int_tol(1e-6);

} // namespace


