/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef P_TRIANGULAR_MATRIX_HPP
#define P_TRIANGULAR_MATRIX_HPP
#include <vector>
#include <algorithm>

namespace p {

// **************************************************************************
// 
// **************************************************************************
template<typename T>
class triangular {

public:
	triangular();
	triangular(int nV);
	triangular(const triangular &src);
	~triangular();
	
	triangular<T> &operator=(const triangular &src);
	
	void clear();
	
	void resize(int nV);
	int  size() const;
	int  elements() const;
	int  offset(int i, int j) const;
	T   &array(int offs = 0);
	const T &array(int offs = 0) const;

	void set(T v);
	void set(int i, int j, T v);
	T    get(int i, int j) const;
	
	T   &operator() (int i, int j);
	T    schur_product(const triangular &src) const;

private:
	int n;
	std::vector<T> values;

};


} // namespace

#endif

