/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PXX_VALUE_HPP
#define PXX_VALUE_HPP
#include <list>
#include <iostream>
#include "timer.hpp"

namespace p {

// ***************************************
class value {

public:
	value();
	value(int identifier, int depth0 = 0);
	value(const std::string &name, int depth0 = 0);
	value(const value &src);
	~value();
	
	value &operator=(const value &src);
	value &operator=(const std::string &v);
	value &operator=(const double &v);
	
	void set_name(const std::string &s);
	const std::string &get_name() const;
	void set_id(int identifier);
	int  get_id() const;
	void set_depth(int d);
	int  get_depth() const;
	
	void clear();
	bool is_double() const;
	bool is_string() const;
	bool is_timer() const;
	bool is_folder() const;
	
	double get_double() const;
	const std::string &get_string() const;
	const char *c_str() const;
	const timer &get_timer() const;

	double conv_double() const;
	std::string conv_string() const;
	
	operator double() const;
	operator std::string() const;
	
	value &operator+=(const std::string &v);
	value &operator+=(const double &v);
	value &operator-=(const double &v);
	value &operator*=(const double &v);
	value &operator/=(const double &v);
	value &operator++();
	int operator++(int);
	value &operator--();
	int operator--(int);
	
	void start();
	void stop();
	void shadow_start();
	void shadow_stop();
	double elapsed_s() const;
	double total_s() const;
	int calls() const;

	value &add_entry(int identifier);
	value &add_entry(const std::string &name);
	value &operator[](int identifier);
	value &operator[](const std::string &name);
	value &operator[](const char *name);
	
	void parse_cmdline(int argc, char **argv);
	
public:
	int id;
	std::string name;
	int depth;
	bool has_dbl, has_str, has_tmr, has_sub;
	
	double dbl;
	std::string str;
	timer tmr;
	std::list<value> l;
};
	
// ***************************************
std::ostream &operator<<(std::ostream &os, const value &v);

extern value values;

// ***************************************

} // namespace

#endif

