/*
    This file is part of pxx -- modeling tools for optimization
    Copyright (C) 2011-2014  Laurent Poirrier

    libp is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with pxx.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PXX_VERTEX_PACKING_HPP
#define PXX_VERTEX_PACKING_HPP
#include <vector>
#include "dense_vector.hpp"
#include "adjacency.hpp"
#include "forest.hpp"

namespace p {

// **************************************************************************
// 
// **************************************************************************
// flags
#define PXX_VP_PERTURB_ND	1
#define PXX_VP_PERTURB_D	2
#define PXX_VP_UNPERTURB_D	4
#define PXX_VP_ENTER_LARGEST	8
#define PXX_VP_ENTER_RANDOM	16
#define PXX_VP_ENTER_ND_EDGE	32
#define PXX_VP_ENTER_ND_NODE	64
#define PXX_VP_ENTER_D_EDGE	128
#define PXX_VP_ENTER_D_NODE	256

class vp {

public:
	vp(adjacency &a, dvec &w);
	~vp();
	
	int operator[] (int j) const;

	bool trivial_infeas(int child) const;
	bool H_condition(int root) const;
	double node_rcost(int j);
	double edge_rcost(int child);

	bool iteration();
	void solve();

	void include_all();
	void exclude(int i, int j);
	bool excluded(int i, int j) const;
	
	void set_flags(int opts);
	void set_max_iter(int iter);
	void set_max_stalls(int iter);
	void set_perturbation(double pmin, double pmax);
	
	double get_objval();
	int get_iter(int flags = 0);
private:
	int n;
	adjacency &a;
	dvec &w;
	forest f;

	struct edge { int i, j; };
	std::vector<vp::edge> ex;
	
	// options
	double pr_min, pr_max;
	int flags;
	int max_iter, max_stalls;
	
	// state
	int iter, iter_nde, iter_ndn, iter_de, iter_dn;
};

// **************************************************************************
// 
// **************************************************************************
} // namespace

#endif

